"""
Note about version numbers:

The version number is tied to the API version of mailchimp that we support.
Version 1.3.0 indicated the first release supporting v1.3 of the API.
"""
__version__ = '1.3.0'



from datetime import datetime

def transform_datetime(dt):
    """ converts datetime parameter"""                               

    if dt is None:
        dt = ''
    else:
        assert isinstance(dt, datetime)
        dt = dt.strftime('%Y-%m-%d %H:%M:%S')
 
    return dt


def flatten(params, key=None):
    """ flatten nested dictionaries and lists """
    flat = {}
    for name, val in params.items():
        if key is not None and not isinstance(key, int):
            name = "%s[%s]" % (key, name)
        if isinstance(val, dict):
            flat.update(flatten(val, name))
        elif isinstance(val, list):
            flat.update(flatten(dict(enumerate(val)), name))
        elif val is not None:
            flat[name] = val
    return flat




import urllib
import datetime
from django.core.urlresolvers import reverse
from django.test import TestCase, RequestFactory
from mailchimp.views import WebHook

class FakeConnection(object):
    def get_list_by_id(self, list_id):
        return None


class NoConnectionWebHook(WebHook):
    WEBHOOK_KEY = '1'
    connection = FakeConnection()


class WebhookTestCase(TestCase):
    fired_at = datetime.datetime(2009, 3, 26, 21, 35, 57)
    groups = ['Group1', 'Group2']
    data = {
        'subscribe': {
            "type": "subscribe",
            "fired_at": "2009-03-26 21:35:57",
            "data[id]": "8a25ff1d98",
            "data[list_id]": "a6b5da1054",
            "data[email]": "api@mailchimp.com",
            "data[email_type]": "html",
            "data[merges][EMAIL]": "api@mailchimp.com",
            "data[merges][FNAME]": "MailChimp",
            "data[merges][LNAME]": "API",
            "data[merges][INTERESTS]": ','.join(groups),
            "data[ip_opt]": "10.20.10.30",
            "data[ip_signup]": "10.20.10.30",
        },
        'unsubscribe': {
            "type": "unsubscribe",
            "fired_at": "2009-03-26 21:40:57",
            "data[action]": "unsub",
            "data[reason]": "manual",
            "data[id]": "8a25ff1d98",
            "data[list_id]": "a6b5da1054",
            "data[email]": "api+unsub@mailchimp.com",
            "data[email_type]": "html",
            "data[merges][EMAIL]": "api+unsub@mailchimp.com",
            "data[merges][FNAME]": "MailChimp",
            "data[merges][LNAME]": "API",
            "data[merges][INTERESTS]": "Group1,Group2",
            "data[ip_opt]": "10.20.10.30",
            "data[campaign_id]": "cb398d21d2",
            "data[reason]": "hard"
        },
        'profile': {
            "type": "profile",
            "fired_at": "2009-03-26 21:31:21",
            "data[id]": "8a25ff1d98",
            "data[list_id]": "a6b5da1054",
            "data[email]": "api@mailchimp.com",
            "data[email_type]": "html",
            "data[merges][EMAIL]": "api@mailchimp.com",
            "data[merges][FNAME]": "MailChimp",
            "data[merges][LNAME]": "API",
            "data[merges][INTERESTS]": "Group1,Group2",
            "data[ip_opt]": "10.20.10.30"
        },
        'upemail': {
            "type": "upemail",
            "fired_at": "2009-03-26\ 22:15:09",
            "data[list_id]": "a6b5da1054",
            "data[new_id]": "51da8c3259",
            "data[new_email]": "api+new@mailchimp.com",
            "data[old_email]": "api+old@mailchimp.com"
        },
        'cleaned': {
            "type": "cleaned",
            "fired_at": "2009-03-26 22:01:00",
            "data[list_id]": "a6b5da1054",
            "data[campaign_id]": "4fjk2ma9xd",
            "data[reason]": "hard",
            "data[email]": "api+cleaned@mailchimp.com"
        }
    }

    def setUp(self):
        self.webhook = NoConnectionWebHook()
        self.url = reverse('mailchimp_webhook', kwargs={'key': '1'})
        self.request_factory = RequestFactory()

    def test_subscribe(self):
        from mailchimp.signals import mc_subscribe
        handler, handler_kwargs = self._get_signal_handler()
        mc_subscribe.connect(handler)
        request = self.request_factory.post(self.url, self.data['subscribe'])
        self.webhook(request, key='1')
        self.assertEqual(handler_kwargs['type'], 'subscribe')
        self.assertEqual(handler_kwargs['fired_at'], self.fired_at)
        self.assertEqual(handler_kwargs['interests'], self.groups)

    def test_unsubscribe(self):
        from mailchimp.signals import mc_unsubscribe
        handler, handler_kwargs = self._get_signal_handler()
        mc_unsubscribe.connect(handler)
        request = self.request_factory.post(self.url, self.data['unsubscribe'])
        self.webhook(request, key='1')
        self.assertEqual(handler_kwargs['type'], 'subscribe')
        self.assertEqual(handler_kwargs['fired_at'], self.fired_at)
        self.assertEqual(handler_kwargs['interests'], self.groups)

    def _get_signal_handler(self):
        handler_kwargs = {}
        def handler(**kwargs):
            handler_kwargs.update(**kwargs)
        return handler, handler_kwargs





from django.core.management import BaseCommand
from django.contrib.sites.models import Site
from mailchimp.utils import get_connection


class Command(BaseCommand):
    def handle(self, *args, **options):
        if len(args) != 1:
            print 'You have to specify exactly one argument to this command'
            return
        merge = args[0]
        print 'Adding the merge var `%s` to all lists' % merge
        c = get_connection()
        for list in c.lists.values():
            print 'Checking list %s' % list.name
            list.add_merges_if_not_exists(merge)
            print '  ok'
        print 'Done'


class ChimpException(Exception): pass

class MCCampaignDoesNotExist(ChimpException): pass
class MCListDoesNotExist(ChimpException): pass
class MCConnectionFailed(ChimpException): pass
class MCTemplateDoesNotExist(ChimpException): pass
class MCFolderDoesNotExist(ChimpException): pass

class MailchimpWarning(Warning): pass


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):
    
    def forwards(self, orm):
        
        # Adding field 'Campaign.object_id'
        db.add_column('mailchimp_campaign', 'object_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True))

        # Adding field 'Campaign.content_type'
        db.add_column('mailchimp_campaign', 'content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True, blank=True))

        # Adding field 'Queue.object_id'
        db.add_column('mailchimp_queue', 'object_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True))

        # Adding field 'Queue.content_type'
        db.add_column('mailchimp_queue', 'content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True, blank=True))
    
    
    def backwards(self, orm):
        
        # Deleting field 'Campaign.object_id'
        db.delete_column('mailchimp_campaign', 'object_id')

        # Deleting field 'Campaign.content_type'
        db.delete_column('mailchimp_campaign', 'content_type_id')

        # Deleting field 'Queue.object_id'
        db.delete_column('mailchimp_queue', 'object_id')

        # Deleting field 'Queue.content_type'
        db.delete_column('mailchimp_queue', 'content_type_id')
    
    
    models = {
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'mailchimp.campaign': {
            'Meta': {'object_name': 'Campaign'},
            'campaign_id': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'sent_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'})
        },
        'mailchimp.queue': {
            'Meta': {'object_name': 'Queue'},
            'authenticate': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'auto_footer': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'auto_tweet': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'campaign_type': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'contents': ('django.db.models.fields.TextField', [], {}),
            'folder_id': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'from_email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'from_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'generate_text': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'google_analytics': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'list_id': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'segment_options': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'segment_options_all': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'segment_options_conditions': ('django.db.models.fields.TextField', [], {}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'template_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'to_email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'tracking_html_clicks': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'tracking_opens': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'tracking_text_clicks': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'type_opts': ('django.db.models.fields.TextField', [], {})
        },
        'mailchimp.reciever': {
            'Meta': {'object_name': 'Reciever'},
            'campaign': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'recievers'", 'to': "orm['mailchimp.Campaign']"}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }
    
    complete_apps = ['mailchimp']



from django.db import models
from django.utils import simplejson
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _
from mailchimp.utils import get_connection


class QueueManager(models.Manager):
    def queue(self, campaign_type, contents, list_id, template_id, subject,
        from_email, from_name, to_email, folder_id=None, tracking_opens=True,
        tracking_html_clicks=True, tracking_text_clicks=False, title=None,
        authenticate=False, google_analytics=None, auto_footer=False,
        auto_tweet=False, segment_options=False, segment_options_all=True,
        segment_options_conditions=[], type_opts={}, obj=None, extra_info=[]):
        """
        Queue a campaign
        """
        kwargs = locals().copy()
        kwargs['segment_options_conditions'] = simplejson.dumps(segment_options_conditions)
        kwargs['type_opts'] = simplejson.dumps(type_opts)
        kwargs['contents'] = simplejson.dumps(contents)
        kwargs['extra_info'] = simplejson.dumps(extra_info)
        for thing in ('template_id', 'list_id'):
            thingy = kwargs[thing]
            if hasattr(thingy, 'id'):
                kwargs[thing] = thingy.id
        del kwargs['self']
        del kwargs['obj']
        if obj:
            kwargs['object_id'] = obj.pk
            kwargs['content_type'] = ContentType.objects.get_for_model(obj)
        return self.create(**kwargs)
    
    def dequeue(self, limit=None):
        if limit:
            qs = self.filter(locked=False)[:limit]
        else:
            qs = self.filter(locked=False)
        for obj in qs:
             yield obj.send()
    
    def get_or_404(self, *args, **kwargs):
        return get_object_or_404(self.model, *args, **kwargs)



class Queue(models.Model):
    """
    A FIFO queue for async sending of campaigns
    """
    campaign_type = models.CharField(max_length=50)
    contents = models.TextField()
    list_id = models.CharField(max_length=50)
    template_id = models.PositiveIntegerField()
    subject = models.CharField(max_length=255)
    from_email = models.EmailField()
    from_name = models.CharField(max_length=255)
    to_email = models.EmailField()
    folder_id = models.CharField(max_length=50, null=True, blank=True)
    tracking_opens = models.BooleanField(default=True)
    tracking_html_clicks = models.BooleanField(default=True)
    tracking_text_clicks = models.BooleanField(default=False)
    title = models.CharField(max_length=255, null=True, blank=True)
    authenticate = models.BooleanField(default=False)
    google_analytics = models.CharField(max_length=100, blank=True, null=True)
    auto_footer = models.BooleanField(default=False)
    generate_text = models.BooleanField(default=False)
    auto_tweet = models.BooleanField(default=False)
    segment_options = models.BooleanField(default=False)
    segment_options_all = models.BooleanField()
    segment_options_conditions = models.TextField()
    type_opts = models.TextField()
    content_type = models.ForeignKey(ContentType, null=True, blank=True)
    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    extra_info = models.TextField(null=True)
    locked = models.BooleanField(default=False)
    
    objects = QueueManager()
    
    def send(self):
        """
        send (schedule) this queued object
        """
        # check lock
        if self.locked:
            return False
        # aquire lock
        self.locked = True
        self.save()
        # get connection and send the mails 
        c = get_connection()
        tpl = c.get_template_by_id(self.template_id)
        content_data = dict([(str(k), v) for k,v in simplejson.loads(self.contents).items()])
        built_template = tpl.build(**content_data)
        tracking = {'opens': self.tracking_opens, 
                    'html_clicks': self.tracking_html_clicks,
                    'text_clicks': self.tracking_text_clicks}
        if self.google_analytics:
            analytics = {'google': self.google_analytics}
        else:
            analytics = {}
        segment_opts = {'match': 'all' if self.segment_options_all else 'any',
            'conditions': simplejson.loads(self.segment_options_conditions)}
        type_opts = simplejson.loads(self.type_opts)
        title = self.title or self.subject
        camp = c.create_campaign(self.campaign_type, c.get_list_by_id(self.list_id),
            built_template, self.subject, self.from_email, self.from_name,
            self.to_email, self.folder_id, tracking, title, self.authenticate,
            analytics, self.auto_footer, self.generate_text, self.auto_tweet,
            segment_opts, type_opts)
        if camp.send_now_async():
            self.delete()
            kwargs = {}
            if self.content_type and self.object_id:
                kwargs['content_type'] = self.content_type
                kwargs['object_id'] = self.object_id
            if self.extra_info:
                kwargs['extra_info'] = simplejson.loads(self.extra_info)
            return Campaign.objects.create(camp.id, segment_opts, **kwargs)
        # release lock if failed
        self.locked = False
        self.save()
        return False
    
    def get_dequeue_url(self):
        return reverse('mailchimp_dequeue', kwargs={'id': self.id})
    
    def get_cancel_url(self):
        return reverse('mailchimp_cancel', kwargs={'id': self.id})
    
    def get_list(self):
        return get_connection().lists[self.list_id]
    
    @property
    def object(self):
        """
        The object might have vanished until now, so triple check that it's there!
        """
        if self.object_id:
            model = self.content_type.model_class()
            try:
                return model.objects.get(id=self.object_id)
            except model.DoesNotExist:
                return None
        return None
    
    def get_object_admin_url(self):
        if not self.object:
            return ''
        name = 'admin:%s_%s_change' % (self.object._meta.app_label,
            self.object._meta.module_name)
        return reverse(name, args=(self.object.pk,))
    
    def can_dequeue(self, user):
        if user.is_superuser:
            return True
        if not user.is_staff:
            return False
        if callable(getattr(self.object, 'mailchimp_can_dequeue', None)):
            return self.object.mailchimp_can_dequeue(user)
        return user.has_perm('mailchimp.can_send') and user.has_perm('mailchimp.can_dequeue') 
    

class CampaignManager(models.Manager):
    def create(self, campaign_id, segment_opts, content_type=None, object_id=None,
            extra_info=[]):
        con = get_connection()
        camp = con.get_campaign_by_id(campaign_id)
        extra_info = simplejson.dumps(extra_info)
        obj = self.model(content=camp.content, campaign_id=campaign_id,
             name=camp.title, content_type=content_type, object_id=object_id,
             extra_info=extra_info)
        obj.save()
        segment_opts = dict([(str(k), v) for k,v in segment_opts.items()])
        for email in camp.list.filter_members(segment_opts):
            Reciever.objects.create(campaign=obj, email=email)
        return obj
    
    def get_or_404(self, *args, **kwargs):
        return get_object_or_404(self.model, *args, **kwargs)
    
    
class DeletedCampaign(object):
    subject = u'<deleted from mailchimp>'


class Campaign(models.Model):
    sent_date = models.DateTimeField(auto_now_add=True)
    campaign_id = models.CharField(max_length=50)
    content = models.TextField()
    name = models.CharField(max_length=255)
    content_type = models.ForeignKey(ContentType, null=True, blank=True)
    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    extra_info = models.TextField(null=True)
    
    objects = CampaignManager()
    
    class Meta:
        ordering = ['-sent_date']
        permissions = [('can_view', 'Can view Mailchimp information'),
                       ('can_send', 'Can send Mailchimp newsletters')]
        verbose_name = _('Mailchimp Log')
        verbose_name_plural = _('Mailchimp Logs')
        
    def get_absolute_url(self):
        return reverse('mailchimp_campaign_info', kwargs={'campaign_id': self.campaign_id})
    
    def get_object_admin_url(self):
        if not self.object:
            return ''
        name = 'admin:%s_%s_change' % (self.object._meta.app_label,
            self.object._meta.module_name)
        return reverse(name, args=(self.object.pk,))
    
    def get_extra_info(self):
        if self.extra_info:
            return simplejson.loads(self.extra_info)
        return []
    
    @property
    def object(self):
        """
        The object might have vanished until now, so triple check that it's there!
        """
        if self.object_id:
            model = self.content_type.model_class()
            try:
                return model.objects.get(id=self.object_id)
            except model.DoesNotExist:
                return None
        return None
    
    @property
    def mc(self):
        try:
            if not hasattr(self, '_mc'):
                self._mc = get_connection().get_campaign_by_id(self.campaign_id)
            return self._mc
        except:
            return DeletedCampaign()


class Reciever(models.Model):
    campaign = models.ForeignKey(Campaign, related_name='recievers')
    email = models.EmailField()


from setuptools import setup, find_packages

version = __import__('mailchimp').__version__

setup(
    name = 'django-mailchimp-v1.3',
    version = version,
    description = 'Mailchimp wrapper for Django, using Mailchimp API 1.3',
    author = 'Jonas Obrist et al.',
    url = 'http://github.com/piquadrat/django-mailchimp',
    packages = find_packages(),
    zip_safe=False,
    package_data={
        'mailchimp': [
            'templates/mailchimp/*.html',
            'locale/*/LC_MESSAGES/*',
        ],
    },
)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from mailchimp.exceptions import MailchimpWarning
import warnings

API_KEY = getattr(settings, 'MAILCHIMP_API_KEY', None)
if API_KEY is None:
    raise ImproperlyConfigured('django-mailchimp requires the MAILCHIMP_API_KEY setting')

SECURE = getattr(settings, 'MAILCHIMP_SECURE', True)

# THIS DOES NOT WORK:
#REAL_CACHE = bool(getattr(settings, 'MAILCHIMP_USE_REAL_CACHE', False))
"""
In [1]: from mailchimp.utils import get_connection

In [2]: c = get_connection ()

In [3]: c.campaigns
key lists <type 'str'>
value {'f48ceea763': <List object: Affichage List>, 'a41f00cba2': <List object: API Test List>} <type 'dict'>
key lists <type 'str'>
value {'f48ceea763': <List object: Affichage List>, 'a41f00cba2': <List object: API Test List>} <type 'dict'>
---------------------------------------------------------------------------
TypeError                                 Traceback (most recent call last)

/home/jonas/workspace/affichage/<ipython console> in <module>()

/home/jonas/workspace/django-mailchimp/mailchimp/chimp.pyc in campaigns(self)
    354     @property
    355     def campaigns(self):
--> 356         return self.get_campaigns()
    357 
    358     def get_campaigns(self):

/home/jonas/workspace/django-mailchimp/mailchimp/chimp.pyc in get_campaigns(self)
    357 
    358     def get_campaigns(self):
--> 359         return self.cache.get('campaigns', self._get_categories)
    360 
    361     @property

/home/jonas/workspace/django-mailchimp/mailchimp/utils.py in get(self, key, obj, *args, **kwargs)
     37         value = self._get(key)
     38         if value is None:
---> 39             value = obj(*args, **kwargs) if callable(obj) else obj
     40             self._set(key, value)
     41         return value

/home/jonas/workspace/django-mailchimp/mailchimp/chimp.pyc in _get_categories(self)
    374 
    375     def _get_categories(self):
--> 376         return build_dict(self, Campaign, self.con.campaigns())
    377 
    378     def _get_lists(self):

/home/jonas/workspace/django-mailchimp/mailchimp/utils.py in build_dict(master, klass, data, key)
     87 
     88 def build_dict(master, klass, data, key='id'):
---> 89     return  dict([(info[key], klass(master, info)) for info in data])
     90 
     91 def _convert(name):

/home/jonas/workspace/django-mailchimp/mailchimp/chimp.pyc in __init__(self, master, info)
     92     def __init__(self, master, info):
     93         super(Campaign, self).__init__(master, info)
---> 94         self.list = self.master.get_list_by_id(self.list_id)
     95         self._content = None
     96         self.frozen_info = info

/home/jonas/workspace/django-mailchimp/mailchimp/chimp.pyc in get_list_by_id(self, id)
    383 
    384     def get_list_by_id(self, id):
--> 385         return self._get_by_id('lists', id)
    386 
    387     def get_campaign_by_id(self, id):

/home/jonas/workspace/django-mailchimp/mailchimp/chimp.pyc in _get_by_id(self, thing, id)
    396     def _get_by_id(self, thing, id):
    397         try:
--> 398             return getattr(self, thing)[id]
    399         except KeyError:
    400             self.cache.flush(thing)

/home/jonas/workspace/django-mailchimp/mailchimp/chimp.pyc in lists(self)
    361     @property
    362     def lists(self):
--> 363         return self.get_lists()
    364 
    365     def get_lists(self):

/home/jonas/workspace/django-mailchimp/mailchimp/chimp.pyc in get_lists(self)
    364 
    365     def get_lists(self):
--> 366         return self.cache.get('lists', self._get_lists)
    367 
    368     @property

/home/jonas/workspace/django-mailchimp/mailchimp/utils.py in get(self, key, obj, *args, **kwargs)
     38         if value is None:
     39             value = obj(*args, **kwargs) if callable(obj) else obj
---> 40             self._set(key, value)
     41         return value
     42 

/home/jonas/workspace/django-mailchimp/mailchimp/utils.py in _real_set(self, key, value)
     44         print 'key', key, type(key)
     45         print 'value', value, type(value)
---> 46         cache.set(key, value, CACHE_TIMEOUT)
     47 
     48     def _real_get(self, key):

/home/jonas/workspace/affichage/parts/django/django/core/cache/backends/locmem.pyc in set(self, key, value, timeout)
     81         try:
     82             try:
---> 83                 self._set(key, pickle.dumps(value), timeout)
     84             except pickle.PickleError:
     85                 pass

/usr/lib/python2.6/copy_reg.pyc in _reduce_ex(self, proto)
     68     else:
     69         if base is self.__class__:
---> 70             raise TypeError, "can't pickle %s objects" % base.__name__
     71         state = base(self)
     72     args = (self.__class__, base, state)

TypeError: can't pickle instancemethod objects

"""
REAL_CACHE = False
CACHE_TIMEOUT = getattr(settings, 'MAILCHIMP_CACHE_TIMEOUT', 300)

WEBHOOK_KEY = getattr(settings, 'MAILCHIMP_WEBHOOK_KEY', '')
if not WEBHOOK_KEY:
    warnings.warn("you did not define a MAILCHIMP_WEBHOOK_KEY setting. "
        "django-mailchimp will create a random one by itself", MailchimpWarning)
    import string
    import random
    alphanum = string.ascii_letters + string.digits
    for x in range(50):
        WEBHOOK_KEY += random.choice(alphanum)

VIEWS_OVERVIEW = getattr(settings, 'MAILCHIMP_VIEWS_OVERVIEW', 'mailchimp.views.overview')
VIEWS_INFO = getattr(settings, 'MAILCHIMP_VIEWS_INFO', 'mailchimp.views.campaign_information')
VIEWS_SCHEDULE_OBJECT = getattr(settings, 'MAILCHIMP_VIEWS_SEND_OBJECT', 'mailchimp.views.schedule_campaign_for_object')
VIEWS_TEST_OBJECT = getattr(settings, 'MAILCHIMP_VIEWS_TEST_OBJECT', 'mailchimp.views.test_campaign_for_object')


from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.views.decorators.csrf import csrf_exempt
from django.http import Http404
from mailchimp.models import Campaign, Queue
from mailchimp.settings import WEBHOOK_KEY
from mailchimp.signals import get_signal
from mailchimp.utils import BaseView, Lazy, get_connection
import datetime
import re

class MailchimpBaseView(BaseView):
    @property
    def connection(self):
        return get_connection()
    
    
class MailchimpView(MailchimpBaseView):
    required_permissions = ['mailchimp.can_view']


class Overview(MailchimpView):
    template = 'mailchimp/overview.html'
    def handle_post(self):
        return self.not_allowed()
    
    def handle_get(self):
        data = {
            'paginator': self.paginate(Campaign.objects.all(), int(self.kwargs.get('page', 1))),
            'queue': Queue.objects.all()
        }
        return self.render_to_response(data)
        
    def get_page_link(self, page):
        return self.reverse('mailchimp_overview', page=page)


class ScheduleCampaignForObject(MailchimpView):
    def auth_check(self):
        basic = super(ScheduleCampaignForObject, self).auth_check()
        if not basic:
            return basic
        return self.request.user.has_perm('mailchimp.can_send')
    
    def handle_post(self):
        return self.not_allowed()
    
    def back(self):
        return self.redirect(self.request.META['HTTP_REFERER'])
    
    def handle_get(self):
        ct = ContentType.objects.get(pk=self.kwargs['content_type'])
        obj = ct.model_class().objects.get(pk=self.kwargs['pk'])
        if obj.mailchimp_schedule(self.connection):
            self.message_success("The Campaign has been scheduled for sending.")
        else:
            self.message_error("An error has occured while trying to send, please try again later.")
        return self.back()
        
        
class TestCampaignForObjectReal(ScheduleCampaignForObject):
    def handle_get(self):
        ct = ContentType.objects.get(pk=self.kwargs['content_type'])
        obj = ct.model_class().objects.get(pk=self.kwargs['pk'])
        self.connection.warnings.reset()
        if obj.mailchimp_test(self.connection, self.request):
            self.message_success("A Test Campaign has been sent to your email address (%s)." % self.request.user.email)
            for message, category, filename, lineno in self.connection.warnings.get():
                self.message_warning("%s: %s" % (category.__name__, message))
        else:
            self.message_error("And error has occured while trying to send the test mail to you, please try again later")
        return self.simplejson(True)
    
    
class TestCampaignForObject(ScheduleCampaignForObject):
    template = 'mailchimp/send_test.html'
    
    def handle_get(self):
        data = {
            'ajaxurl': reverse('mailchimp_real_test_for_object', kwargs=self.kwargs),
            'redirecturl': self.request.META['HTTP_REFERER']
        }
        return self.render_to_response(data)


class CampaignInformation(MailchimpView):
    template = 'mailchimp/campaign_information.html'
    def handle_post(self):
        return self.not_allowed()
    
    def handle_get(self):
        camp = Campaign.objects.get_or_404(campaign_id=self.kwargs['campaign_id'])
        data = {'campaign': camp}
        extra_info = camp.get_extra_info()
        if camp.object and hasattr(camp.object, 'mailchimp_get_extra_info'):
            extra_info = camp.object.mailchimp_get_extra_info()
        data['extra_info'] = extra_info
        return self.render_to_response(data)
        

class WebHook(MailchimpBaseView):
    def handle_get(self):
        return self.response("hello chimp")
    
    def handle_post(self):
        if self.kwargs.get('key', '') != WEBHOOK_KEY:
            return self.not_found()
        data = self.request.POST
        signal = get_signal(data['type'])
        ts = data["fired_at"]
        fired_at = datetime.datetime.strptime(ts, "%Y-%m-%d %H:%M:%S")
        kwargs = {
            'list': self.connection.get_list_by_id(data["data[list_id]"]),
            'fired_at': fired_at,
            'type': data['type'],
        }
        if data['type'] == 'cleaned':
            kwargs.update({
                'reason': data['data[reason]'],
                'email': data['data[email]'],
            })
        elif data['type'] == 'upemail':
            kwargs.update({
                 'old_email': data['data[old_email]'],
                 'new_email': data['data[new_email]'],
            })
        elif data['type'] == 'campaign':
            kwargs.update({
                'campaign_id': data['data[id]'],
                'subject': data['data[subject]'],
                'status': data['data[status]'],
                'reason': data['data[reason]'],
            })
        else:
            merge_re = re.compile('data\[merges\]\[(?P<name>w+)\]')
            merges = {}
            for key, value in data.items():
                match = merge_re.match(key)
                if match:
                    name = match.group('name').lower()
                    if name in ('interests', 'fname', 'lname'):
                        continue
                    merges[name] = value
            kwargs.update({
                'email': data['data[email]'],
                'fname': data['data[merges][FNAME]'],
                'lname': data['data[merges][LNAME]'],
                'merges': merges,
            })
            if 'data[merges][INTERESTS]' in data:
                kwargs['interests'] = [i.strip() for i in data['data[merges][INTERESTS]'].split(',')]
        signal.send(sender=self.connection, **kwargs)
        return self.response("ok")

        
class Dequeue(ScheduleCampaignForObject):
    def handle_get(self):
        q = Queue.objects.get_or_404(pk=self.kwargs['id'])
        if q.send():
            self.message_success("The Campaign has successfully been dequeued.")
        else:
            self.message_error("An error has occured while trying to dequeue this campaign, please try again later.")
        return self.back()
        

class Cancel(ScheduleCampaignForObject):
    def handle_get(self):
        q = Queue.objects.get_or_404(pk=self.kwargs['id'])
        q.delete()
        self.message_success("The Campaign has been canceled.")
        return self.back()


webhook = csrf_exempt(WebHook())
dequeue = Dequeue()
cancel = Cancel()
campaign_information = CampaignInformation()
overview = Overview()
schedule_campaign_for_object = ScheduleCampaignForObject()
test_campaign_for_object = TestCampaignForObject()
test_real = TestCampaignForObjectReal()


"""
Example cronjob:

from cronjobs.base import Cron
from cronjobs.constants import MINUTES
from mailchimp.utils import dequeue

class DequeueCron(Cron):
    run_every = 1
    interval_unit = MINUTES
 
    def job(self):
        try:
            dequeue()
            return True
        except:
            return False
"""